#include <cassert>
#include <iostream>
#include <random>
#include <string>

// This program demonstrates how to generate random integers using the
// STL random number generator


int main(int argc, char* argv[]) {

  if (argc != 3) {
    std::cerr << "ERROR: This program expects 2 arguments:" << std::endl;
    std::cerr << "the # of values to generate and the max integer value." << std::endl;
    return 1;
  }

  int number = std::stoi(argv[1]);
  int max = std::stoi(argv[2]);
  assert (number > 0);
  assert (max > 0);


  std::cout << "Generating " << number 
            << " random integer values in the range [0," << max << "]" << std::endl;


  // Using a FIXED SEED can be useful for testing & debugging.  This
  // sequence will be the same on future runs of the program with the
  // same arguments (a.k.a. deterministic).
  std::default_random_engine fixed_random_generator;
  std::cout << "fixed seed:             ";
  for (int i = 0; i < number; i++) {
    int x = fixed_random_generator() % (max+1);
    std::cout << " " << x;
  }  
  std::cout << std::endl;


  // "SEEDING" the random number generator will start the generation
  // at a different point in the sequence.  Here we use a built in OS
  // specific random device (often based on the current clock).  This
  // sequence will be different of future runs of the program.
  std::random_device r;
  std::default_random_engine seeded_random_generator(r());
    
  std::cout << "random device seed:     ";
  for (int i = 0; i < number; i++) {
    int x = seeded_random_generator() % (max+1);
    std::cout << " " << x;
  }  
  std::cout << std::endl;


}
